/* -*- mode: c++; c-basic-offset: 3; -*- */
#include "PConfig.h"
#include "matchInterface.hh"
#include "sockutil.h"
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <cstring>
#include <unistd.h>
#include <iostream>
#include <iomanip>
using namespace std;

// #define MIF_DEBUG 1

namespace framexmit {
   //===================================  if_list constructors
   if_list::if_list(int s) {
      if (s<0) return;
      getInterfaces(s);
   }

   //===================================  if_list constructors
   if_list::if_list(void) {
      int s = socket(AF_INET, SOCK_DGRAM, 0);
      if (s<0) return;
      getInterfaces(s);
      close(s);
   }

   //===================================  Print out the list of interfaces
   void
   if_list::display(std::ostream& out) const {
      out << "List of available devices" << hex << endl;
      for (auto i=iList.begin(); i != iList.end(); i++) {
         out << i->name << "  " << i->addr << "  " << i->mask << endl;
      }
      out << endl;
    }
   //===================================  Get a list of interfaces.
   bool 
   if_list::getInterfaces(int sock)  {
      // clear list
      iList.clear();

      // get list of interfaces
      ifconf 	ifc;
      char 	buf[2048];
      ifc.ifc_len = sizeof (buf);
      ifc.ifc_buf = buf;
      if (ioctl (sock, SIOCGIFCONF, (char*) &ifc) < 0) {
         return false;
      }
   
      // obtaining interface flags
      ifreq*	ifr = ifc.ifc_req;
      for (int n = ifc.ifc_len / sizeof (ifreq); --n >=0; ifr++) {
         // inet interfaces only
         if (ifr->ifr_addr.sa_family != AF_INET) {
            continue;
         }

	 //-----------------------------  get flags
         if (ioctl (sock, SIOCGIFFLAGS, (char*) ifr) < 0) {
            return -1;
         }

	 //-----------------------------  skip uninteresting cases
         if (((ifr->ifr_flags & IFF_UP) == 0) ||
	     ((ifr->ifr_flags & IFF_LOOPBACK) != 0) ||
	     ((ifr->ifr_flags & (IFF_BROADCAST | IFF_POINTOPOINT)) == 0)) {
            continue;
         }

         //-----------------------------  fill in name and address
         interface_type iface;
         strcpy (iface.name, ifr->ifr_name);
         iface.addr = ((sockaddr_in*)(&ifr->ifr_addr))->sin_addr.s_addr;

         //-----------------------------  Get the net mask
         if (ioctl (sock, SIOCGIFNETMASK, (char*) ifr) < 0) {
            return -1;
         }
         iface.mask = ((sockaddr_in*)(&ifr->ifr_netmask))->sin_addr.s_addr;

         //-----------------------------  Get the net mask
         if (ioctl (sock, SIOCGIFBRDADDR, (char*) ifr) < 0) {
            return -1;
         }
         iface.brdaddr = ((sockaddr_in*)(&ifr->ifr_broadaddr))->sin_addr.s_addr;

         //-----------------------------   Add to the list
         iList.push_back (iface);
      }
      return true;
   }

   //===================================  get subnet from addr and mask
   inline uint32_t
   subnet(uint32_t addr, uint32_t mask) {
      return ntohl(addr) & ntohl(mask);
   }

   //===================================  Match interface to network.
   if_list::const_iflist_iter
   if_list::match(const char* net) const {
     in_addr_t net_ip(0);
     if (net)  net_ip = ntohl(inet_addr(net));
      for (const_iflist_iter i = iList.begin(); i != iList.end(); i++) {
         if (subnet(i->addr, i->mask) == subnet(htonl(net_ip), i->mask)) {
            return i;
         }
      }
      return end();
   }


   //===================================  Match interface to network.
   bool 
   matchInterface (int sock, const char* net, in_addr& i_addr) {
     //--------------------------------  Get net address in host ordering
     in_addr_t net_ip(~0);
     if (net)  net_ip = ntohl(inet_addr(net));

     //--------------------------------  If net address not specified
     if ( net_ip == in_addr_t(~0) ) {
        // use default host address
        char           myname[256];    // local host name
        if (gethostname(myname, sizeof(myname)) || nslookup(myname, &i_addr)){
           return false;
        }
     }

     //--------------------------------  Search for specified address.
     else {
        // get a list of all interface addresses
        if_list iList(sock);
        if (iList.empty()) {
           return false;
        }

        //-----------------------------  Find a match
        auto i = iList.match(net);
        if (i == iList.end()) return false;
        i_addr.s_addr = i->addr;
     }
     return true;
  }

} // namespace framexmit
